### R code from vignette source 'vignette.Rnw'

###################################################
### code chunk number 1: SourceR
###################################################
library(favir)
InitPaper()


###################################################
### code chunk number 2: LatexPrelude
###################################################
IncludePrelude("Marginal Likelihood Computation with the \\texttt{margLikArrogance} package", "Benedict Escoto", header.lines="\\usepackage{amssymb,amsmath}
\\newcommand{\\bx}{\\boldsymbol x}
\\newcommand{\\btheta}{\\boldsymbol \\theta}",
               include.logo=FALSE)


###################################################
### code chunk number 3: codeSample
###################################################
t1.mean.mean <- 5
t1.mean.sd <- 0.1
t1.sd <- 0.5

t2.mu.mean <- 1.6
t2.mu.sd <- .02
t2.sigma <- .1

t3.mean.mean <- 5
t3.mean.sd <- .1
t3.prec.rate <- 4 / 1^2
t3.prec.shape <- 4 * t3.prec.rate

set.seed(1)
t1.prior.samples <- rnorm(1000,
                          mean=rnorm(1000, mean=t1.mean.mean, sd=t1.mean.sd),
                          sd=t1.sd)
t2.prior.samples <- exp(rnorm(1000,
                              rnorm(1000, mean=t2.mu.mean, sd=t2.mu.sd),
                              t2.sigma))
t3.prior.mu.samples <- rnorm(1000, mean=t3.mean.mean, sd=t3.mean.sd)
t3.prior.prec.samples <- rgamma(1000, shape=t3.prec.shape, rate=t3.prec.rate)
t3.prior.samples <- rnorm(1000, mean=t3.prior.mu.samples,
                          sd=1 / sqrt(t3.prior.prec.samples))
p1.df <- data.frame(x=c(t1.prior.samples, t2.prior.samples, t3.prior.samples),
                    t=c(rep("Theory 1", 1000), rep("Theory 2", 1000),
                        rep("Theory 3", 1000)))
p1 <- (ggplot(data=p1.df)
       + geom_histogram(aes(x=x), binwidth=.2)
       + facet_wrap(~ t, ncol=1)
       + labs(x="Height", y="Count"))


###################################################
### code chunk number 4: samplePlot
###################################################
IncludeGraph(p1, caption="Prior Marginal Distributions", label="prior")


###################################################
### code chunk number 5: heightsTable
###################################################
set.seed(1)
n <- 60
samples <- rnorm(n, mean=5.1, sd=0.7)
sample.table.df <- data.frame(p1=1:15, h1=samples[1:15],
                              p2=16:30, h2=samples[16:30],
                              p3=31:45, h3=samples[31:45],
                              p4=46:60, h4=samples[46:60])
sample.fdf <- FavirDF(sample.table.df,
                      label="height samples",
                      caption="Sampled Heights")
FieldHeadings(sample.fdf) <- c(p1="Person \\#", p2="Person \\#",
                               p3="Person \\#", p4="Person \\#",
                               h1="Height", h2="Height",
                               h3="Height", h4="Height")
FieldFormatters(sample.fdf) <- list(h1=formatters$flat2, h2=formatters$flat2,
                                    h3=formatters$flat2, h4=formatters$flat2)
print(sample.fdf)


###################################################
### code chunk number 6: conjugateCode
###################################################
UpdateMean <- function(mean.mean, mean.sd, sd, x) {
  # Return posterior mean and standard deviation of normal mean conjugate
  post.sd <- sqrt(1 / (1 / mean.sd^2 + length(x) / sd^2))
  return(c(mean=(mean.mean / mean.sd^2 + sum(x) / sd^2) * post.sd^2,
           sd=post.sd))
}

set.seed(1)
t1.post.param <- UpdateMean(t1.mean.mean, t1.mean.sd, t1.sd, samples)
t1.post.samples <- rnorm(1000, mean=t1.post.param["mean"],
                         sd=t1.post.param["sd"])

t2.post.param <- UpdateMean(t2.mu.mean, t2.mu.sd, t2.sigma, log(samples))
t2.post.samples <- rnorm(1000, mean=t2.post.param["mean"],
                         sd=t2.post.param["sd"])


###################################################
### code chunk number 7: gibbsSampler
###################################################
UpdatePrecision <- function(prec.shape, prec.rate, mean, x) {
  # Return posterior shape and rate of inverse gamma precision
  return(c(shape=prec.shape + length(x) / 2,
           rate=prec.rate + sum((x - mean)^2) / 2))
}

t3.mean.post.samples <- rep(NA, 1010)
t3.sd.post.samples <- rep(NA, 1010)

t3.mean.post.samples[1] <- 5
t3.sd.post.samples[1] <- 0.5

set.seed(1)
for (i in 2:1010) {
  mu.params <- UpdateMean(t3.mean.mean, t3.mean.sd, t3.sd.post.samples[i-1],
                          samples)
  t3.mean.post.samples[i] <- rnorm(1, mu.params["mean"], mu.params["sd"])
  prec.params <- UpdatePrecision(t3.prec.shape, t3.prec.rate,
                                 t3.mean.post.samples[i-1], samples)
  t3.sd.post.samples[i] <- sqrt(1/rgamma(1, shape=prec.params["shape"],
                                         rate=prec.params["rate"]))
}
t3.post.samples <- cbind(t3.mean.post.samples, t3.sd.post.samples)[11:1010, ]
colnames(t3.post.samples) <- c("mean", "sd")


###################################################
### code chunk number 8: paramComparison
###################################################
param.comp.df <- data.frame(mean=c(t3.prior.mu.samples,
                                   t3.post.samples[, "mean"]),
                            sd=c(1/sqrt(t3.prior.prec.samples),
                                 t3.post.samples[, "sd"]),
                            type=c(rep("Prior", 1000), rep("Posterior", 1000)),
                            stringsAsFactors=FALSE, check.rows=TRUE)
pcomp.plot <- (ggplot(data=param.comp.df)
               + geom_point(aes(x=mean, y=sd), alpha=0.4)
               + facet_wrap(~ type, ncol=1)
               + labs(x="Mean", y="Standard Deviation"))
IncludeGraph(pcomp.plot, caption="Theory 3 Prior vs Posterior Parameters",
             label="parameter comparison")


###################################################
### code chunk number 9: T1LogLik
###################################################
T1OneLik <- function(theta) {
  # Compute log likelihood for one sample from T1's posterior param dist
  ll <- sum(dnorm(samples, mean=theta, sd=t1.sd, log=TRUE))
  return(ll + dnorm(theta, mean=t1.mean.mean, sd=t1.mean.sd, log=TRUE))
}
t1.ll <- sapply(t1.post.samples, T1OneLik)


###################################################
### code chunk number 10: T2LogLik
###################################################
T2OneLik <- function(theta) {
  # Compute log likelihood for one sample from T2's posterior param dist
  ll <- sum(dlnorm(samples, meanlog=theta, sdlog=t2.sigma, log=TRUE))
  return(ll + dnorm(theta, mean=t2.mu.mean, sd=t2.mu.sd, log=TRUE))
}
t2.ll <- sapply(t2.post.samples, T2OneLik)


###################################################
### code chunk number 11: T3LogLik
###################################################
T3OneLik <- function(theta) {
  # Compute log likelihood for one sample from T3's posterior param dist
  ll <- sum(dnorm(samples, mean=theta[1], sd=theta[2], log=TRUE))
  param.ll <- (dnorm(theta[1], mean=t3.mean.mean, sd=t3.mean.sd, log=TRUE)
               + dgamma(1 / theta[2]^2, shape=t3.prec.shape,
                        rate=t3.prec.rate, log=TRUE))
  return(ll + param.ll)
}
t3.ll <- apply(t3.post.samples, 1, T3OneLik)


###################################################
### code chunk number 12: ComputeLikelihoods
###################################################
library(margLikArrogance)
t1.margll <- MarginalLikelihood(matrix(t1.post.samples, ncol=1), t1.ll)
t2.margll <- MarginalLikelihood(matrix(t2.post.samples, ncol=1), t2.ll)
t3.margll <- MarginalLikelihood(t3.post.samples, t3.ll,
                                bounds=matrix(c(-Inf, Inf, 0.1, Inf), nrow=2))


###################################################
### code chunk number 13: results
###################################################
conf.str <- c(paste(formatC(t1.margll$conf.interval, digits=1, format="f"),
                    collapse=" to "),
              paste(formatC(t2.margll$conf.interval, digits=1, format="f"),
                    collapse=" to "),
              paste(formatC(t3.margll$conf.interval, digits=1, format="f"),
                    collapse=" to "))

t1.actual.mll <- (t1.ll[1]
                  - dnorm(t1.post.samples[1], mean=t1.post.param["mean"],
                          sd=t1.post.param["sd"], log=TRUE))
t2.actual.mll <- (t2.ll[2]
                  - dnorm(t2.post.samples[2], mean=t2.post.param["mean"],
                          sd=t2.post.param["sd"], log=TRUE))

result.df <- data.frame(theory=1:3,
                        mll.est=c(t1.margll$mll, t2.margll$mll, t3.margll$mll),
                        conf=conf.str,
                        mll.actual=c(t1.actual.mll, t2.actual.mll, NA))
avg.ll <- sum(exp(result.df$mll.est)) / 3
result.df$post.prob <- exp(result.df$mll.est) / avg.ll / 3

result.fdf <- FavirDF(result.df,
                      label="results", caption="Arrogance Sampling Results")
FieldHeadings(result.fdf) <- c(theory="Theory",
                               mll.est="Estimated Marg LL",
                               conf="Confidence Interval",
                               mll.actual="Actual MLL (Analytic)",
                               post.prob="Posterior Probability")
FieldFormatters(result.fdf) <- list(theory=formatters$flat0,
                                    mll.est=formatters$flat1,
                                    mll.actual=formatters$flat1,
                                    post.prob=formatters$flat2)
print(result.fdf)


###################################################
### code chunk number 14: guidelines
###################################################
guidelines.df <- data.frame(diff=c("0 to 1", "1 to 3", "3 to 5", "> 5"),
                            desc=c("Inconsequential", "Positive", "Strong",
                                   "Very Strong"))
guidelines.fdf <- FavirDF(guidelines.df, label="guidelines",
                          caption="Kass and Raftery's Guidelines")
FieldHeadings(guidelines.fdf) <- c(diff="LL Difference",
                                   desc="Strength of Evidence")
print(guidelines.fdf)


###################################################
### code chunk number 15: Ex2
###################################################
library(mvtnorm)
mvn.n <- 10^5
mvn.mu1 <- c(1, rep(0, 14))
mvn.mu2 <- 10^6 * mvn.mu1
set.seed(1)
mvn.theta <- rbind(rmvnorm(mvn.n/2, mean=mvn.mu1),
                   rmvnorm(mvn.n/2, mean=mvn.mu2))


###################################################
### code chunk number 16: Ex2ll
###################################################
mvn.ll <- (1000 + log(dmvnorm(mvn.theta, mean=mvn.mu1) / 2
                      + dmvnorm(mvn.theta, mean=mvn.mu2) / 2))


###################################################
### code chunk number 17: Ex2answer
###################################################
mvn.mll <- MarginalLikelihood(mvn.theta, mvn.ll)
mvn.mll$mll
mvn.mll$conf.interval


